@@ -145,6 +145,7 @@ import Utility.Exception
 -- | Controls how a region is laid out in the console.
 --
 -- Here's an annotated example of how the console layout works.
+-- Each sequence of the same letter represents a distinct region.
 --
 -- > scrolling......
 -- > scrolling......
@@ -258,10 +259,26 @@ instance ToRegionContent (STM Text) where
 -- necessary.
 --
 -- The value can include ANSI SGR escape sequences for changing
--- the colors etc of all or part of a region.
+-- the colors of all or part of a region. For this to display properly,
+-- a reset escape sequence must be included to get the color back
+-- to default. System.Console.ANSI makes it easy to construct such
+-- values. For example:
+--
+-- > import System.Console.ANSI
+-- > 
+-- > setConsoleRegion region 
+-- > 	( "hello "
+-- > 	<> setSGRCode [SetColor Foreground Vivid Red] 
+-- >	<> "Mars" 
+-- >	<> setSGRCode [Reset]
+-- > 	<> "!"
+-- >	)
 -- 
 -- Other ANSI escape sequences, especially those doing cursor
 -- movement, will mess up the layouts of regions. Caveat emptor.
+--
+-- ANSI SGR escape sequences that span multiple lines do not currently
+-- display as you might hope. (Patches would be accepted.)
 setConsoleRegion :: (ToRegionContent v, LiftRegion m) => ConsoleRegion -> v -> m ()
 setConsoleRegion r v = liftRegion $
 	modifyRegion r $ const $ pure $ toRegionContent v
@@ -295,7 +312,9 @@ resizeRegion width (ConsoleRegion tv) = do
 -- | Runs the action with a new console region, closing the region when
 -- the action finishes or on exception.
 withConsoleRegion :: (MonadIO m, MonadMask m) => RegionLayout -> (ConsoleRegion -> m a) -> m a
-withConsoleRegion ly = bracketIO (openConsoleRegion ly) (closeConsoleRegion)
+withConsoleRegion ly = bracketIO
+	(openConsoleRegion ly)
+	(uninterruptibleMask_ . closeConsoleRegion)
 
 -- | Opens a new console region.
 openConsoleRegion :: LiftRegion m => RegionLayout -> m ConsoleRegion
@@ -404,7 +423,9 @@ removeChild child _parent@(ConsoleRegion pv) = do
 -- Note that this uses `lockOutput`, so it takes over all output to the
 -- console while the passed IO action is running. As well as displaying
 -- the console regions, this handles display of anything buffered by
--- `outputConcurrent` and `createProcessConcurrent`.
+-- `outputConcurrent` and `createProcessConcurrent`. So,
+-- `withConcurrentOutput` and `flushConcurrentOutput` should not be run
+-- while this is in use, and will block.
 --
 -- When standard output is not an ANSI capable terminal,
 -- console regions are not displayed.
@@ -414,17 +435,17 @@ displayConsoleRegions a = ifM (liftIO regionDisplayEnabled)
 	, lockOutput $ bracket setup cleanup (const a)
 	)
   where
-	setup = liftIO $ do
+	setup = liftIO $ uninterruptibleMask $ \unmask -> do
 		atomically $ putTMVar regionList []
 		endsignal <- atomically $ do
 			s <- newTSem 1
 			waitTSem s
 			return s
 		isterm <- liftIO $ hSupportsANSI stdout
-		when isterm trackConsoleWidth
-		da <- async $ displayThread isterm endsignal
+		when isterm (unmask trackConsoleWidth)
+		da <- async $ unmask $ displayThread isterm endsignal
 		return (isterm, da, endsignal)
-	cleanup (isterm, da, endsignal) = liftIO $ do
+	cleanup (isterm, da, endsignal) = liftIO $ uninterruptibleMask_ $ do
 		atomically $ signalTSem endsignal
 		void $ wait da
 		void $ atomically $ takeTMVar regionList
@@ -467,7 +488,7 @@ waitDisplayChange :: STM a -> IO a
 waitDisplayChange a = do
 	c <- atomically $ dupTChan displayUpdateNotifier
 	bv <- newEmptyTMVarIO
-	setbarrier bv `concurrently` waitchange c bv
+	_ <- setbarrier bv `concurrently` waitchange c bv
 	snd <$> atomically (readTMVar bv)
   where
 	setbarrier bv = atomically $ do
